/*
 * UNC001 firmware. (c) Unicontrollers, Ltd., 2010
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file _mkusbid.c
 * This program is used to generate a part of the C code (the file
 * _usb_id.h); see also the detailed comment below.
 *
 * Written by Andrey V. Stolyarov
 *
 */



/* This program is the simplest way to translate the USB identification
 * data from the simple form given in ../unc_id.h into the form required by
 * the driver's usbconfig.h
 *
 * Presently, it doesn't support non-printable characters such as tabs and
 * newlines within the identification strings; I didn't spend the time on
 * this because it is not needed in the current project version.
 */

#include <stdio.h>
#include "unc_id.h"

static void output_atomized_string(const char *str)
{
    int i;
    for(;*str;) {
        printf("        ");
        for(i=0; i<16; i++) {
            switch(*str) {
            case '\'':
                printf("\'\\\'\'");
                break;
            case '\t':
                printf("\'\\t\'");
                break;
            case '\n':
            case '\r':
            case '\b':
                printf("\'?\'");
                break;
            default:
                printf("\'%c\'", *str);
            }
            str++;
            if(!*str) break;
            printf(",");
        }
        if(*str) printf(" \\\n");
    }
    printf("\n");
}

int main(int argc, char **argv)
{
    printf("/* DO NOT EDIT THIS FILE */\n");
    printf("/* GENERATED by %s */\n\n", argv[0]);

    printf("#define USB_CFG_VENDOR_ID 0x%02x, 0x%02x\n",
           UNC_VENDOR_ID % 0x100, UNC_VENDOR_ID / 0x100);
    printf("#define USB_CFG_DEVICE_ID 0x%02x, 0x%02x\n",
           UNC_DEVICE_ID % 0x100, UNC_DEVICE_ID / 0x100);
    printf("#define USB_CFG_DEVICE_VERSION 0x%02x, 0x%02x\n",
           UNC_DEVICE_VERSION % 0x100, UNC_DEVICE_VERSION / 0x100);

    printf("\n");

    printf("#define USB_CFG_VENDOR_NAME \\\n");
    output_atomized_string(UNC_VENDOR_NAME);
    printf("#define USB_CFG_VENDOR_NAME_LEN %d\n", sizeof(UNC_VENDOR_NAME)-1);

    printf("\n");

    printf("#define USB_CFG_DEVICE_NAME \\\n");
    output_atomized_string(UNC_DEVICE_NAME);
    printf("#define USB_CFG_DEVICE_NAME_LEN %d\n", sizeof(UNC_DEVICE_NAME)-1);

    return 0;
}
