/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012-2014
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file config.hpp
 * Class headers for the host-side configuration management
 *
 * Written by Andrey V. Stolyarov
 *
 */

#ifndef CONFIG_HPP_SENTRY
#define CONFIG_HPP_SENTRY

enum device_visibility {
    devv_always_visible,
    devv_default,
    devv_hidden,
    devv_ignored
};

enum line_type {
    lnt_none = 0,
    lnt_input = 0x04,
    lnt_output = 0x0f
};

class DeviceConfig {
    friend DeviceConfig* GetDeviceConfig(int);
    int id;
    char strid[10];
private:
    DeviceConfig(int aid);
    DeviceConfig(const DeviceConfig &) {} // prohibited
    void operator=(const DeviceConfig &) {} // prohibited
    ~DeviceConfig() {}
public:
    const int GetId() const { return id; }
    const char *GetName() const;
    device_visibility GetVisibility() const;

    const char *GetLineName(int n) const;
    bool IsButtonSticky(int n) const;
    bool InputAlert(int n) const;
    line_type GetModuleType(int n) const; // n is 0 or 1
    line_type GetLineType(int n) const;   // n is 0 .. 11

    void SetName(const char *name);
    void SetLineName(int n, const char *name);
    void SetButtonSticky(int n, bool sticky);
    void SetInputAlert(int n, bool alert);
    void SetModuleType(int n, line_type t, bool seen);
    void SetVisibility(device_visibility dv);

    bool Save();
};

struct ForciblyShownDevices {
    int count;
    int *array;
    ForciblyShownDevices();
    ~ForciblyShownDevices();
private:
    ForciblyShownDevices(const ForciblyShownDevices&) {} // prohibited
    void operator=(const ForciblyShownDevices&) {} // prohibited
};


bool InitConfig(const char *fname);
DeviceConfig* GetDeviceConfig(int);

#endif
