/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012-2014
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file demo_dev.cpp
 * Communication with the device for GUI demo program
 *
 * Written by Andrey V. Stolyarov
 *
 */

#include <stdio.h>

#include <unc0xx.h>
#include <unc_data.h>

#include "demo_dev.hpp"


UNCDevice::~UNCDevice()
{
    if(dev_hdl)
        unc0xx_close(dev_hdl);
}

bool UNCDevice::OpenDevice()
{
    if(dev_hdl) {
        unc0xx_close(dev_hdl);
        dev_hdl = 0;
    }
    struct unc0xx_scan_item *list;
    unc0xx_usb_init();
    list = unc0xx_scan(id);
    if(!list)
        return false;
    dev_hdl = list->dev_hdl;
    list->dev_hdl = 0;
    unc0xx_destroy_list(list);
    return true;
}

static void
process_1w_item(uint8_t *id, int itemp, char temp[32], char tabl[32])
{
    int j, grad, migrad;
    char *p;
    char tsign = '+';
    if(!id[0])
        return;
    switch(itemp) {
    case (short)0x8000: /* no temperature here => tablet? */
        if(id[0] != 1) {
            // not a tablet
            break;
        }
        p = tabl;
        sprintf(p, "%02X:", id[7]);
        p+=3;
        for(j=6; j>=1; j--) {
            sprintf(p, "%02X", id[j]);
            p += 2;
        }
        sprintf(p, ":%02X", id[0]);
        p+=3;
        *p = 0;
        break;
    case 0x7fff: /* crc error */
        break;
    default:
        grad = itemp >> 4;
        migrad = (itemp & 0x0f) * 625;
        if(temp < 0) {
            tsign = '-';
            if(migrad > 0) {
                migrad = 10000 - migrad;
                grad = -grad - 1;
            } else {
                grad = -grad;
            }
        }
        sprintf(temp, "%c%d.%04d C", tsign, grad, migrad);
    }
}


bool UNCDevice::Poll(int *state, char temp[32], char tabl[32])
{
    if(!dev_hdl) return false;

    struct unc0xx_global_state_str devstat;
    struct unc0xx_extra_state_str_common extra;
    int r;

    r = unc0xx_get_fullstate(dev_hdl, &devstat);
    if(r == -1) {
        return false;
    }
    *state = devstat.state_bits;

    temp[0] = 0;
    tabl[0] = 0;
    r = unc0xx_get_extrastate(dev_hdl, &extra);
    if(r != -1) {
        if(extra.page_version == 0) {
            unc0xx_extra_state_str_legacy *ex =
                (unc0xx_extra_state_str_legacy*)&extra; 
            for(int i=0; i<4; i++)
                process_1w_item(ex->ow_ids[i], ex->ow_temperature[i],
                                temp, tabl);
        } else
        if(extra.page_version == 1) {
            unc0xx_extra_state_str_onewire *ex =
                (unc0xx_extra_state_str_onewire*)&extra; 
            for(int i=0; i<6; i++)
                process_1w_item(ex->ow_ids[i], ex->ow_temperature[i],
                                temp, tabl);
        } 
    }
    return true;
}

void UNCDevice::ToggleLine(int n)
{
    int action;
    if(n < 4) {
        action = 0x03 << (2*n);
    } else if(n >= 8) {
        action = 0x0200 | (0x03 << (2*(n-8)));
    } else {
        return ; // no operation
    }
    PerformAction(action);
}

void UNCDevice::PerformAction(int action)
{
    if(!dev_hdl) return;
    unc0xx_perform_action(dev_hdl, action);
}
