/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file gui.cpp
 * Implementation of the GUI application and its main window
 *
 * Written by Andrey V. Stolyarov
 *
 */

#include <wx/frame.h>
#include <wx/sizer.h>
#include <wx/gbsizer.h>
#include <wx/stattext.h>
#include <wx/statbox.h>
#include <wx/tglbtn.h>
#include <wx/button.h>
#include <wx/checkbox.h>
#include <wx/gauge.h>
#include <wx/timer.h>
#include <wx/msgdlg.h>
#include <wx/textdlg.h>

#include <stdlib.h>

#include "config.hpp"
#include "devopdlg.hpp"
#include "hwcfgdlg.hpp"

#include "gui.hpp"



SingleDevicePanel::
SingleDevicePanel(ScrolledDeviceList *parent, int a_id)
    : wxPanel(parent), UncCtrlsetInterface(parent, a_id), the_parent(parent) 
{
    int i;
    for(i=0; i<12; i++) outs[i] = 0;
    for(i=0; i<12; i++) sticky[i] = 0;
    for(i=0; i<8; i++) delims[i] = 0;
    for(i=0; i<8; i++) count[i] = 0;
    for(i=0; i<8; i++) ins[i] = 0;
    for(i=0; i<8; i++) innames[i] = 0;
    for(i=0; i<8; i++) alert[i] = 0;

    DeviceConfig *conf = GetDeviceConfig(Id());
    statbs = new wxStaticBoxSizer(wxVERTICAL, this,
        wxString::Format(wxT("[%04x] "), Id()) +
        wxString(conf->GetName(), wxConvLocal) +
        wxT(" {not connected}")
    );
    grid = new wxGridBagSizer(2, 2);
    grid->SetRows(14);
    grid->SetCols(6);
    statbs->Add(grid, 0, wxEXPAND, 0);

    grid->SetFlexibleDirection(wxHORIZONTAL);
    grid->AddGrowableCol(0);
    grid->AddGrowableCol(4);
    grid->Add(new wxStaticText(this, wxID_STATIC, wxT("Relays")));
    // grid->Add(new wxStaticText(this, wxID_STATIC, wxT("Sticky")));
    grid->Add(new wxStaticText(this, wxID_STATIC, wxT("")));
    grid->Add(new wxStaticText(this, wxID_STATIC, wxT("")));
    zero = new wxButton(this, wid_zero, wxT(">0<"),
                        wxDefaultPosition, wxDefaultSize,
                        wxBU_EXACTFIT);
    grid->Add(zero);
    grid->Add(new wxStaticText(this, wxID_STATIC, wxT("Inputs")));
#if 0
    grid->Add(new wxStaticText(this, wxID_STATIC, wxT("Alert")));
#endif

    statusbar = new wxStaticText(this, wxID_STATIC, wxT(""));
    statbs->Add(statusbar, 0, wxEXPAND); 

    Reconfigure();

    Connect(wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,
            wxCommandEventHandler(SingleDevicePanel::OnOutputButton));

    wxBoxSizer* butsz = new wxBoxSizer(wxHORIZONTAL);
    options = new wxButton(this, wid_options, wxT("Options"));
    butsz->Add(options);
    hwopts = new wxButton(this, wid_hwopts, wxT("Tune device"));
    butsz->Add(hwopts);
    dismiss = new wxButton(this, wid_dismiss, wxT("Hide"));
    butsz->Add(dismiss);
    
    statbs->Add(butsz, 0, wxALIGN_RIGHT);

    statbs->SetSizeHints(this);
    SetSizer(statbs);
}

void SingleDevicePanel::Reconfigure()
{
    int i;
#define GET_RID_OF(x) \
    if(x[i]) {              \
        grid->Detach(x[i]); \
        delete x[i];        \
        x[i] = 0;           \
    }

    for(i=0; i<12; i++) {
        GET_RID_OF(outs)
        GET_RID_OF(sticky)
    }
    for(i=0; i<8; i++) {
        GET_RID_OF(delims)
        GET_RID_OF(count)
        GET_RID_OF(ins)
        GET_RID_OF(innames)
        GET_RID_OF(alert)
    }
#undef GET_RID_OF

    DeviceConfig *conf = GetDeviceConfig(Id());

    UpdateConnectedState();

    int busy_left = 1, busy_right = 1;
    int rcol = 1;
    for(int i=0; i<12; i++) {
        switch(conf->GetLineType(i)) {
        case lnt_none:
            continue;
        case lnt_input:
            delims[i-4] = new wxStaticText(this, wxID_STATIC, wxT(" | "));
            grid->Add(delims[i-4], wxGBPosition(busy_right, rcol));
            ins[i-4] = new wxGauge(this, wxID_STATIC, 100,
                               wxDefaultPosition, wxSize(20,20));
            grid->Add(ins[i-4], wxGBPosition(busy_right, rcol+1));
            count[i-4] = new wxStaticText(this, wxID_STATIC, wxT("[0]"));
            grid->Add(count[i-4], wxGBPosition(busy_right, rcol+2));
            innames[i-4] = new wxStaticText(this, wxID_STATIC,
                           wxString(conf->GetLineName(i), wxConvLocal));
            grid->Add(innames[i-4], wxGBPosition(busy_right, rcol+3),
                      wxDefaultSpan, wxGROW);
#if 0
            alert[i-4] = new wxCheckBox(this, wid_in_alert, wxT(""));
            grid->Add(alert[i-4], wxGBPosition(busy_right, rcol+4));
#endif
            busy_right++;
            break;
        case lnt_output:
            outs[i] = new wxToggleButton(this, wid_out_button + i, 
                           wxString(conf->GetLineName(i), wxConvLocal));
            grid->Add(outs[i], wxGBPosition(busy_left, 0),
                      wxDefaultSpan, wxGROW);
            //sticky[i] = new wxCheckBox(this, wid_out_sticky + i, wxT(""));
            //grid->Add(sticky[i]);
            //grid->AddSpacer(20);
            busy_left++;
        }
    }
    grid->Layout();
    statbs->Layout();
    statbs->SetSizeHints(this);
    statbs->Fit(this);
    the_parent->Rearrange();
}

void SingleDevicePanel::OnOutputButton(wxCommandEvent& event)
{
    int id = event.GetId();
    if(id >= wid_out_button && id < (wid_out_button + series_wid)) {
        // this is an output button event
        if(the_device)
            the_device->SetLine(id - wid_out_button, event.GetInt()==1);
    } else {
        event.Skip();
    }
}

void SingleDevicePanel::OnDismiss(wxCommandEvent& event)
{
    GetDeviceConfig(Id())->
        SetVisibility(the_device ? devv_hidden : devv_default);
    GetDeviceConfig(Id())->Save();
    ScrolledDeviceList *p = the_parent;
    delete this;
    p->Rearrange();
}

void SingleDevicePanel::OnZero(wxCommandEvent& event)
{
    if(the_device) the_device->ZeroCounters();
}

void SingleDevicePanel::OnOptions(wxCommandEvent& event)
{
    LaunchDeviceOptionsDialog(Id(), the_parent);
}

void SingleDevicePanel::OnHwOpts(wxCommandEvent& event)
{
    LaunchHardwareConfigurationDialog(Id(),
                                      the_parent->GetDeviceCollection());
}

void SingleDevicePanel::SetInput(int line, bool state)
{
    if(line < 0 || line > 7) return;
    if(!ins[line]) return;
#if 0
    ins[line]->SetBackgroundColour(wxTheColourDatabase->Find(
                             state ? wxT("AQUAMARINE") : wxT("GRAY")));
#else
    ins[line]->SetValue(state ? 100 : 0);
#endif
    innames[line]->SetForegroundColour(wxTheColourDatabase->Find(
                             state ? wxT("RED") : wxT("BLACK")));
}

void SingleDevicePanel::SetOutput(int line, bool state)
{
    if(line < 0 || line > 12) return;
    if(!outs[line]) return; // shouldn't happen, actually
    outs[line]->SetValue(state);
}

void SingleDevicePanel::SetCounter(int line, int value)
{
    if(line < 0 || line > 7) return;
    if(!count[line]) return; // shouldn't happen, actually
    count[line]->SetLabel(wxString::Format(wxT("[%d]"), value));
}

void SingleDevicePanel::SetStatusBar(const char *status)
{
    statusbar->SetLabel(wxString(status, wxConvLocal));
#ifndef _WIN32
    statbs->Layout();
    statbs->SetSizeHints(this);
    statbs->Fit(this);
    the_parent->Rearrange();
#endif
}

void SingleDevicePanel::UpdateConnectedState()
{
    DeviceConfig *conf = GetDeviceConfig(Id());
    statbs->GetStaticBox()->SetLabel(
        wxString::Format(wxT("[%04x] "), Id()) +
        wxString(conf->GetName(), wxConvLocal) +
        (the_device ? wxT("") : wxT(" {not connected}"))
    );
    Refresh();
}

BEGIN_EVENT_TABLE(SingleDevicePanel, wxPanel)
  EVT_BUTTON(wid_zero,    SingleDevicePanel::OnZero)
  EVT_BUTTON(wid_options, SingleDevicePanel::OnOptions)
  EVT_BUTTON(wid_hwopts,  SingleDevicePanel::OnHwOpts)
  EVT_BUTTON(wid_dismiss, SingleDevicePanel::OnDismiss)
END_EVENT_TABLE()


ScrolledDeviceList::ScrolledDeviceList(wxWindow* parent, wxWindowID id, 
                                       UncDeviceCollection *a_devcoll)
    : wxScrolledWindow(parent, id), UncCtrlsetCollection(a_devcoll),
      mins(wxDefaultSize), sizer(0)
{
    sizer = new wxBoxSizer(wxVERTICAL);
    
    SetSizer(sizer);
    SetScrollRate(5, 5);
}

UncCtrlsetInterface* ScrolledDeviceList::AddDevice(int id)
{
    SingleDevicePanel *panel = new SingleDevicePanel(this, id);
    mins = panel->GetEffectiveMinSize();

    sizer->Add(panel, wxSizerFlags().Expand());
    Rearrange();

    return panel;
}

void ScrolledDeviceList::Rearrange()
{
    sizer->Layout();  // this is the sacred knowledge!
    FitInside();      // this is to make up the scrollers
}

UncCtrlsetInterface* ScrolledDeviceList::Provide(int id)
{
    Iterator iter(this);
    UncCtrlsetInterface *c;
    while((c = iter.Next())) {
        if(c->Id() == id)
            return c;
    }
    return AddDevice(id);
}


bool UncMonitorGui::OnInit()
{
    if(!wxApp::OnInit())
        return false;

    frame = new wxFrame((wxFrame *)NULL, -1, wxT("UNC Monitor"));
    
    wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
    list = new ScrolledDeviceList(frame, wxID_ANY, the_devcoll);
    sizer->Add(list, 1, wxEXPAND);

    //sizer->Add(new wxStaticLine(frame, -1));
    sizer->AddSpacer(5);

    wxBoxSizer* butsz = new wxBoxSizer(wxHORIZONTAL);
    
    adddev = new wxButton(frame, wxID_ADD, wxT("Add device"));
    butsz->Add(adddev);
    rescan = new wxButton(frame, wxID_REFRESH, wxT("Rescan"));
    butsz->Add(rescan);
    about = new wxButton(frame, wxID_ABOUT, wxT("About"));
    butsz->Add(about);
    quit = new wxButton(frame, wxID_EXIT, wxT("Quit"));
    butsz->Add(quit);

    sizer->Add(butsz, 0, wxALIGN_RIGHT);

    //frame->SetSizerAndFit(sizer);
    frame->SetSizer(sizer);
    //sizer->SetSizeHints(frame);
    frame->SetSize(wxSize(600, 300));
    frame->Show(false);
    frame->Show(true);

    the_devcoll->SetCtrlsetCollection(list);
    the_devcoll->Rescan(false);

    return true;
} 

void UncMonitorGui::OnAdddev(wxCommandEvent& event)
{
    wxString res = wxGetTextFromUser(
        wxT("Please enter the device Id\nExample: 7775"),
        wxT("Adding device..."));
    if(res == wxT("")) return;
    long id;
    if(!res.ToLong(&id, 16)) {
        wxMessageBox(wxT("Invalid id"), wxT("Error"), wxOK|wxICON_ERROR);
        return;
    }
    GetDeviceConfig(id)->SetVisibility(devv_always_visible);
    list->Provide(id);
}

void UncMonitorGui::OnRescan(wxCommandEvent& event)
{
    the_devcoll->Rescan(true);
}


void LaunchAboutDialog(); /* from the aboutdlg.cpp file */

void UncMonitorGui::OnAbout(wxCommandEvent& event)
{
    LaunchAboutDialog();
}

void UncMonitorGui::OnQuit(wxCommandEvent& event)
{
    frame->Close(true);
    exit(0);
}

BEGIN_EVENT_TABLE(UncMonitorGui, wxApp)
  EVT_BUTTON(wxID_ADD,     UncMonitorGui::OnAdddev)
  EVT_BUTTON(wxID_REFRESH, UncMonitorGui::OnRescan)
  EVT_BUTTON(wxID_ABOUT,   UncMonitorGui::OnAbout)
  EVT_BUTTON(wxID_EXIT,    UncMonitorGui::OnQuit)
END_EVENT_TABLE()

/*wid_rescan*/

class PollingTimer : public wxTimer {
    UncDeviceCollection *the_coll;
    int interval;
public:
    PollingTimer(UncDeviceCollection *a_coll, int ainterv)
        : the_coll(a_coll), interval(ainterv)
    {}
    void Go() {
        Start(interval);
    }
    void Notify() {
        the_coll->Poll();
    }
};



static UncMonitorGui *the_gui = 0;
static PollingTimer *the_timer = 0;

IMPLEMENT_WX_THEME_SUPPORT

void uncmon_gui_init(class UncDeviceCollection *devcoll)
{
    the_gui = new UncMonitorGui(devcoll);
    wxApp::SetInstance(the_gui);
    the_timer = new PollingTimer(devcoll, 200 /* XXX must be adjustable */);
}

void uncmon_gui_run(const char *appname)
{
    char *argv_for_wx[2];
    int argc_for_wx = 1;

    the_timer->Go();

    argv_for_wx[0] = (char *)appname;
    argv_for_wx[1] = 0;
    // I don't know why this takes the arguments, and I'm not going to
    // pass the real argv to it as I prefer to handle them myself, NOT
    // delegating the task to (heh) the GUI library.  Would be too stupid.
    wxEntry(argc_for_wx, argv_for_wx);
}

void uncmon_gui_cleanup()
{
}
